pub type IUIAnimationInterpolator = *mut ::core::ffi::c_void;
pub type IUIAnimationInterpolator2 = *mut ::core::ffi::c_void;
pub type IUIAnimationLoopIterationChangeHandler2 = *mut ::core::ffi::c_void;
pub type IUIAnimationManager = *mut ::core::ffi::c_void;
pub type IUIAnimationManager2 = *mut ::core::ffi::c_void;
pub type IUIAnimationManagerEventHandler = *mut ::core::ffi::c_void;
pub type IUIAnimationManagerEventHandler2 = *mut ::core::ffi::c_void;
pub type IUIAnimationPrimitiveInterpolation = *mut ::core::ffi::c_void;
pub type IUIAnimationPriorityComparison = *mut ::core::ffi::c_void;
pub type IUIAnimationPriorityComparison2 = *mut ::core::ffi::c_void;
pub type IUIAnimationStoryboard = *mut ::core::ffi::c_void;
pub type IUIAnimationStoryboard2 = *mut ::core::ffi::c_void;
pub type IUIAnimationStoryboardEventHandler = *mut ::core::ffi::c_void;
pub type IUIAnimationStoryboardEventHandler2 = *mut ::core::ffi::c_void;
pub type IUIAnimationTimer = *mut ::core::ffi::c_void;
pub type IUIAnimationTimerClientEventHandler = *mut ::core::ffi::c_void;
pub type IUIAnimationTimerEventHandler = *mut ::core::ffi::c_void;
pub type IUIAnimationTimerUpdateHandler = *mut ::core::ffi::c_void;
pub type IUIAnimationTransition = *mut ::core::ffi::c_void;
pub type IUIAnimationTransition2 = *mut ::core::ffi::c_void;
pub type IUIAnimationTransitionFactory = *mut ::core::ffi::c_void;
pub type IUIAnimationTransitionFactory2 = *mut ::core::ffi::c_void;
pub type IUIAnimationTransitionLibrary = *mut ::core::ffi::c_void;
pub type IUIAnimationTransitionLibrary2 = *mut ::core::ffi::c_void;
pub type IUIAnimationVariable = *mut ::core::ffi::c_void;
pub type IUIAnimationVariable2 = *mut ::core::ffi::c_void;
pub type IUIAnimationVariableChangeHandler = *mut ::core::ffi::c_void;
pub type IUIAnimationVariableChangeHandler2 = *mut ::core::ffi::c_void;
pub type IUIAnimationVariableCurveChangeHandler2 = *mut ::core::ffi::c_void;
pub type IUIAnimationVariableIntegerChangeHandler = *mut ::core::ffi::c_void;
pub type IUIAnimationVariableIntegerChangeHandler2 = *mut ::core::ffi::c_void;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UIAnimationManager: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x4c1fc63a_695c_47e8_a339_1a194be3d0b8);
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UIAnimationManager2: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xd25d8842_8884_4a4a_b321_091314379bdd);
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UIAnimationTimer: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xbfcd4a0c_06b6_4384_b768_0daa792c380e);
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UIAnimationTransitionFactory: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x8a9b1cdd_fcd7_419c_8b44_42fd17db1887);
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UIAnimationTransitionFactory2: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x84302f97_7f7b_4040_b190_72ac9d18e420);
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UIAnimationTransitionLibrary: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x1d6322ad_aa85_4ef5_a828_86d71067d145);
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UIAnimationTransitionLibrary2: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x812f944a_c5c8_4cd9_b0a6_b3da802f228d);
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_REPEAT_INDEFINITELY: i32 = -1i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_REPEAT_INDEFINITELY_CONCLUDE_AT_END: i32 = -1i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_REPEAT_INDEFINITELY_CONCLUDE_AT_START: i32 = -2i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_SECONDS_EVENTUALLY: i32 = -1i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_SECONDS_INFINITE: i32 = -1i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub type UI_ANIMATION_DEPENDENCIES = u32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_DEPENDENCY_NONE: UI_ANIMATION_DEPENDENCIES = 0u32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_DEPENDENCY_INTERMEDIATE_VALUES: UI_ANIMATION_DEPENDENCIES = 1u32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_DEPENDENCY_FINAL_VALUE: UI_ANIMATION_DEPENDENCIES = 2u32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_DEPENDENCY_FINAL_VELOCITY: UI_ANIMATION_DEPENDENCIES = 4u32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_DEPENDENCY_DURATION: UI_ANIMATION_DEPENDENCIES = 8u32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub type UI_ANIMATION_IDLE_BEHAVIOR = i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_IDLE_BEHAVIOR_CONTINUE: UI_ANIMATION_IDLE_BEHAVIOR = 0i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_IDLE_BEHAVIOR_DISABLE: UI_ANIMATION_IDLE_BEHAVIOR = 1i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub type UI_ANIMATION_MANAGER_STATUS = i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_MANAGER_IDLE: UI_ANIMATION_MANAGER_STATUS = 0i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_MANAGER_BUSY: UI_ANIMATION_MANAGER_STATUS = 1i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub type UI_ANIMATION_MODE = i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_MODE_DISABLED: UI_ANIMATION_MODE = 0i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_MODE_SYSTEM_DEFAULT: UI_ANIMATION_MODE = 1i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_MODE_ENABLED: UI_ANIMATION_MODE = 2i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub type UI_ANIMATION_PRIORITY_EFFECT = i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_PRIORITY_EFFECT_FAILURE: UI_ANIMATION_PRIORITY_EFFECT = 0i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_PRIORITY_EFFECT_DELAY: UI_ANIMATION_PRIORITY_EFFECT = 1i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub type UI_ANIMATION_REPEAT_MODE = i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_REPEAT_MODE_NORMAL: UI_ANIMATION_REPEAT_MODE = 0i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_REPEAT_MODE_ALTERNATE: UI_ANIMATION_REPEAT_MODE = 1i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub type UI_ANIMATION_ROUNDING_MODE = i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_ROUNDING_NEAREST: UI_ANIMATION_ROUNDING_MODE = 0i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_ROUNDING_FLOOR: UI_ANIMATION_ROUNDING_MODE = 1i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_ROUNDING_CEILING: UI_ANIMATION_ROUNDING_MODE = 2i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub type UI_ANIMATION_SCHEDULING_RESULT = i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_SCHEDULING_UNEXPECTED_FAILURE: UI_ANIMATION_SCHEDULING_RESULT = 0i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_SCHEDULING_INSUFFICIENT_PRIORITY: UI_ANIMATION_SCHEDULING_RESULT = 1i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_SCHEDULING_ALREADY_SCHEDULED: UI_ANIMATION_SCHEDULING_RESULT = 2i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_SCHEDULING_SUCCEEDED: UI_ANIMATION_SCHEDULING_RESULT = 3i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_SCHEDULING_DEFERRED: UI_ANIMATION_SCHEDULING_RESULT = 4i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub type UI_ANIMATION_SLOPE = i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_SLOPE_INCREASING: UI_ANIMATION_SLOPE = 0i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_SLOPE_DECREASING: UI_ANIMATION_SLOPE = 1i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub type UI_ANIMATION_STORYBOARD_STATUS = i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_STORYBOARD_BUILDING: UI_ANIMATION_STORYBOARD_STATUS = 0i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_STORYBOARD_SCHEDULED: UI_ANIMATION_STORYBOARD_STATUS = 1i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_STORYBOARD_CANCELLED: UI_ANIMATION_STORYBOARD_STATUS = 2i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_STORYBOARD_PLAYING: UI_ANIMATION_STORYBOARD_STATUS = 3i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_STORYBOARD_TRUNCATED: UI_ANIMATION_STORYBOARD_STATUS = 4i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_STORYBOARD_FINISHED: UI_ANIMATION_STORYBOARD_STATUS = 5i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_STORYBOARD_READY: UI_ANIMATION_STORYBOARD_STATUS = 6i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_STORYBOARD_INSUFFICIENT_PRIORITY: UI_ANIMATION_STORYBOARD_STATUS = 7i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub type UI_ANIMATION_TIMER_CLIENT_STATUS = i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_TIMER_CLIENT_IDLE: UI_ANIMATION_TIMER_CLIENT_STATUS = 0i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_TIMER_CLIENT_BUSY: UI_ANIMATION_TIMER_CLIENT_STATUS = 1i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub type UI_ANIMATION_UPDATE_RESULT = i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_UPDATE_NO_CHANGE: UI_ANIMATION_UPDATE_RESULT = 0i32;
#[doc = "*Required features: `\"Win32_UI_Animation\"`*"]
pub const UI_ANIMATION_UPDATE_VARIABLES_CHANGED: UI_ANIMATION_UPDATE_RESULT = 1i32;
pub type UI_ANIMATION_KEYFRAME = isize;
