pub type ICompositionCapabilitiesInteropFactory = *mut ::core::ffi::c_void;
pub type ICompositionDrawingSurfaceInterop = *mut ::core::ffi::c_void;
pub type ICompositionDrawingSurfaceInterop2 = *mut ::core::ffi::c_void;
pub type ICompositionGraphicsDeviceInterop = *mut ::core::ffi::c_void;
pub type ICompositorDesktopInterop = *mut ::core::ffi::c_void;
pub type ICompositorInterop = *mut ::core::ffi::c_void;
pub type IDesktopWindowTargetInterop = *mut ::core::ffi::c_void;
pub type ISwapChainInterop = *mut ::core::ffi::c_void;
pub type IVisualInteractionSourceInterop = *mut ::core::ffi::c_void;
