#include <stdarg.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdlib.h>

#define SIGNED_DOESNT_NEED_ULL_SUFFIX -9223372036854775807

#define SIGNED_NEEDS_ULL_SUFFIX -9223372036854775808ULL

#define UNSIGNED_DOESNT_NEED_ULL_SUFFIX 8070450532247928832

#define UNSIGNED_NEEDS_ULL_SUFFIX 9223372036854775808ULL
