// This file is generated by cbindgen. DO NOT EDIT

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

void root(void);

#ifdef __cplusplus
} // extern "C"
#endif // __cplusplus

/* This is a simple test to ensure that trailers don't cause extra newlines in files */
