#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DevQuery_ObjectType: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 325533506, data2: 41942, data3: 18934, data4: [180, 218, 174, 70, 224, 197, 35, 124] }, pid: 2u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceClass_Characteristics: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1126273419, data2: 63134, data3: 18189, data4: [165, 222, 77, 136, 199, 90, 210, 75] }, pid: 29u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceClass_ClassCoInstallers: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1899828995, data2: 41698, data3: 18933, data4: [146, 20, 86, 71, 46, 243, 218, 92] }, pid: 2u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceClass_ClassInstaller: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 630898684, data2: 20647, data3: 18382, data4: [175, 8, 104, 201, 167, 215, 51, 102] }, pid: 5u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceClass_ClassName: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 630898684, data2: 20647, data3: 18382, data4: [175, 8, 104, 201, 167, 215, 51, 102] }, pid: 3u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceClass_DHPRebalanceOptOut: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 3511500531, data2: 26319, data3: 19362, data4: [157, 56, 13, 219, 55, 171, 71, 1] }, pid: 2u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceClass_DefaultService: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 630898684, data2: 20647, data3: 18382, data4: [175, 8, 104, 201, 167, 215, 51, 102] }, pid: 11u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceClass_DevType: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1126273419, data2: 63134, data3: 18189, data4: [165, 222, 77, 136, 199, 90, 210, 75] }, pid: 27u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceClass_Exclusive: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1126273419, data2: 63134, data3: 18189, data4: [165, 222, 77, 136, 199, 90, 210, 75] }, pid: 28u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceClass_Icon: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 630898684, data2: 20647, data3: 18382, data4: [175, 8, 104, 201, 167, 215, 51, 102] }, pid: 4u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceClass_IconPath: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 630898684, data2: 20647, data3: 18382, data4: [175, 8, 104, 201, 167, 215, 51, 102] }, pid: 12u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceClass_LowerFilters: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1126273419, data2: 63134, data3: 18189, data4: [165, 222, 77, 136, 199, 90, 210, 75] }, pid: 20u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceClass_Name: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 630898684, data2: 20647, data3: 18382, data4: [175, 8, 104, 201, 167, 215, 51, 102] }, pid: 2u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceClass_NoDisplayClass: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 630898684, data2: 20647, data3: 18382, data4: [175, 8, 104, 201, 167, 215, 51, 102] }, pid: 8u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceClass_NoInstallClass: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 630898684, data2: 20647, data3: 18382, data4: [175, 8, 104, 201, 167, 215, 51, 102] }, pid: 7u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceClass_NoUseClass: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 630898684, data2: 20647, data3: 18382, data4: [175, 8, 104, 201, 167, 215, 51, 102] }, pid: 10u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceClass_PropPageProvider: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 630898684, data2: 20647, data3: 18382, data4: [175, 8, 104, 201, 167, 215, 51, 102] }, pid: 6u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceClass_Security: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1126273419, data2: 63134, data3: 18189, data4: [165, 222, 77, 136, 199, 90, 210, 75] }, pid: 25u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceClass_SecuritySDS: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1126273419, data2: 63134, data3: 18189, data4: [165, 222, 77, 136, 199, 90, 210, 75] }, pid: 26u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceClass_SilentInstall: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 630898684, data2: 20647, data3: 18382, data4: [175, 8, 104, 201, 167, 215, 51, 102] }, pid: 9u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceClass_UpperFilters: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1126273419, data2: 63134, data3: 18189, data4: [165, 222, 77, 136, 199, 90, 210, 75] }, pid: 19u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_Address: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 51u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_AlwaysShowDeviceAsConnected: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 101u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_AssociationArray: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 80u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_BaselineExperienceId: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 78u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_Category: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 90u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_CategoryGroup_Desc: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 94u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_CategoryGroup_Icon: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 95u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_Category_Desc_Plural: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 92u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_Category_Desc_Singular: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 91u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_Category_Icon: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 93u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_ConfigFlags: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 105u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_CustomPrivilegedPackageFamilyNames: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 107u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_DeviceDescription1: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 81u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_DeviceDescription2: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 82u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_DeviceFunctionSubRank: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 100u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_DiscoveryMethod: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 52u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_ExperienceId: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 89u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_FriendlyName: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1701460915, data2: 60608, data3: 17405, data4: [132, 119, 74, 224, 64, 74, 150, 205] }, pid: 12288u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_HasProblem: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 83u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_Icon: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 57u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_InstallInProgress: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2212127526, data2: 38822, data3: 16520, data4: [148, 83, 161, 146, 63, 87, 59, 41] }, pid: 9u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_IsAuthenticated: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 54u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_IsConnected: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 55u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_IsDefaultDevice: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 86u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_IsDeviceUniquelyIdentifiable: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 79u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_IsEncrypted: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 53u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_IsLocalMachine: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 70u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_IsMetadataSearchInProgress: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 72u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_IsNetworkDevice: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 85u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_IsNotInterestingForDisplay: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 74u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_IsPaired: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 56u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_IsRebootRequired: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 108u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_IsSharedDevice: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 84u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_IsShowInDisconnectedState: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 68u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_Last_Connected: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 67u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_Last_Seen: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 66u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_LaunchDeviceStageFromExplorer: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 77u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_LaunchDeviceStageOnDeviceConnect: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 76u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_Manufacturer: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1701460915, data2: 60608, data3: 17405, data4: [132, 119, 74, 224, 64, 74, 150, 205] }, pid: 8192u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_MetadataCabinet: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 87u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_MetadataChecksum: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 73u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_MetadataPath: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 71u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_ModelName: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1701460915, data2: 60608, data3: 17405, data4: [132, 119, 74, 224, 64, 74, 150, 205] }, pid: 8194u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_ModelNumber: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1701460915, data2: 60608, data3: 17405, data4: [132, 119, 74, 224, 64, 74, 150, 205] }, pid: 8195u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_PrimaryCategory: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 97u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_PrivilegedPackageFamilyNames: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 106u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_RequiresPairingElevation: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 88u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_RequiresUninstallElevation: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 99u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_UnpairUninstall: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 98u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceContainer_Version: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 65u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceInterfaceClass_DefaultInterface: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 348666521, data2: 2879, data3: 17591, data4: [190, 76, 161, 120, 211, 153, 5, 100] }, pid: 2u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceInterfaceClass_Name: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 348666521, data2: 2879, data3: 17591, data4: [190, 76, 161, 120, 211, 153, 5, 100] }, pid: 3u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceInterface_Autoplay_Silent: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1129173647, data2: 40565, data3: 17674, data4: [154, 185, 255, 97, 230, 24, 186, 208] }, pid: 2u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceInterface_ClassGuid: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 40784238, data2: 47124, data3: 16715, data4: [131, 205, 133, 109, 111, 239, 72, 34] }, pid: 4u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceInterface_Enabled: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 40784238, data2: 47124, data3: 16715, data4: [131, 205, 133, 109, 111, 239, 72, 34] }, pid: 3u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceInterface_FriendlyName: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 40784238, data2: 47124, data3: 16715, data4: [131, 205, 133, 109, 111, 239, 72, 34] }, pid: 2u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceInterface_ReferenceString: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 40784238, data2: 47124, data3: 16715, data4: [131, 205, 133, 109, 111, 239, 72, 34] }, pid: 5u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceInterface_Restricted: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 40784238, data2: 47124, data3: 16715, data4: [131, 205, 133, 109, 111, 239, 72, 34] }, pid: 6u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceInterface_SchematicName: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 40784238, data2: 47124, data3: 16715, data4: [131, 205, 133, 109, 111, 239, 72, 34] }, pid: 9u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DeviceInterface_UnrestrictedAppCapabilities: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 40784238, data2: 47124, data3: 16715, data4: [131, 205, 133, 109, 111, 239, 72, 34] }, pid: 8u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_AdditionalSoftwareRequested: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2830656989, data2: 11837, data3: 16532, data4: [173, 151, 229, 147, 167, 12, 117, 214] }, pid: 19u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_Address: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 30u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_AssignedToGuest: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1410045054, data2: 35648, data3: 17852, data4: [168, 162, 106, 11, 137, 76, 189, 162] }, pid: 24u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_BaseContainerId: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 38u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_BiosDeviceName: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1410045054, data2: 35648, data3: 17852, data4: [168, 162, 106, 11, 137, 76, 189, 162] }, pid: 10u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_BusNumber: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 23u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_BusRelations: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1128310469, data2: 37882, data3: 18182, data4: [151, 44, 123, 100, 128, 8, 165, 167] }, pid: 7u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_BusReportedDeviceDesc: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1410045054, data2: 35648, data3: 17852, data4: [168, 162, 106, 11, 137, 76, 189, 162] }, pid: 4u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_BusTypeGuid: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 21u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_Capabilities: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 17u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_Characteristics: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 29u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_Children: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1128310469, data2: 37882, data3: 18182, data4: [151, 44, 123, 100, 128, 8, 165, 167] }, pid: 9u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_Class: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 9u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_ClassGuid: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 10u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_CompatibleIds: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 4u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_ConfigFlags: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 12u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_ConfigurationId: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1410045054, data2: 35648, data3: 17852, data4: [168, 162, 106, 11, 137, 76, 189, 162] }, pid: 7u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_ContainerId: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2357121542, data2: 16266, data3: 18471, data4: [179, 171, 174, 158, 31, 174, 252, 108] }, pid: 2u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_CreatorProcessId: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1410045054, data2: 35648, data3: 17852, data4: [168, 162, 106, 11, 137, 76, 189, 162] }, pid: 25u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_DHP_Rebalance_Policy: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1410045054, data2: 35648, data3: 17852, data4: [168, 162, 106, 11, 137, 76, 189, 162] }, pid: 2u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_DebuggerSafe: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1410045054, data2: 35648, data3: 17852, data4: [168, 162, 106, 11, 137, 76, 189, 162] }, pid: 12u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_DependencyDependents: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1410045054, data2: 35648, data3: 17852, data4: [168, 162, 106, 11, 137, 76, 189, 162] }, pid: 21u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_DependencyProviders: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1410045054, data2: 35648, data3: 17852, data4: [168, 162, 106, 11, 137, 76, 189, 162] }, pid: 20u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_DevNodeStatus: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1128310469, data2: 37882, data3: 18182, data4: [151, 44, 123, 100, 128, 8, 165, 167] }, pid: 2u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_DevType: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 27u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_DeviceDesc: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 2u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_Driver: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 11u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_DriverCoInstallers: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2830656989, data2: 11837, data3: 16532, data4: [173, 151, 229, 147, 167, 12, 117, 214] }, pid: 11u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_DriverDate: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2830656989, data2: 11837, data3: 16532, data4: [173, 151, 229, 147, 167, 12, 117, 214] }, pid: 2u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_DriverDesc: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2830656989, data2: 11837, data3: 16532, data4: [173, 151, 229, 147, 167, 12, 117, 214] }, pid: 4u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_DriverInfPath: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2830656989, data2: 11837, data3: 16532, data4: [173, 151, 229, 147, 167, 12, 117, 214] }, pid: 5u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_DriverInfSection: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2830656989, data2: 11837, data3: 16532, data4: [173, 151, 229, 147, 167, 12, 117, 214] }, pid: 6u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_DriverInfSectionExt: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2830656989, data2: 11837, data3: 16532, data4: [173, 151, 229, 147, 167, 12, 117, 214] }, pid: 7u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_DriverLogoLevel: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2830656989, data2: 11837, data3: 16532, data4: [173, 151, 229, 147, 167, 12, 117, 214] }, pid: 15u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_DriverProblemDesc: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1410045054, data2: 35648, data3: 17852, data4: [168, 162, 106, 11, 137, 76, 189, 162] }, pid: 11u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_DriverPropPageProvider: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2830656989, data2: 11837, data3: 16532, data4: [173, 151, 229, 147, 167, 12, 117, 214] }, pid: 10u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_DriverProvider: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2830656989, data2: 11837, data3: 16532, data4: [173, 151, 229, 147, 167, 12, 117, 214] }, pid: 9u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_DriverRank: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2830656989, data2: 11837, data3: 16532, data4: [173, 151, 229, 147, 167, 12, 117, 214] }, pid: 14u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_DriverVersion: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2830656989, data2: 11837, data3: 16532, data4: [173, 151, 229, 147, 167, 12, 117, 214] }, pid: 3u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_EjectionRelations: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1128310469, data2: 37882, data3: 18182, data4: [151, 44, 123, 100, 128, 8, 165, 167] }, pid: 4u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_EnumeratorName: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 24u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_Exclusive: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 28u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_ExtendedAddress: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1410045054, data2: 35648, data3: 17852, data4: [168, 162, 106, 11, 137, 76, 189, 162] }, pid: 23u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_ExtendedConfigurationIds: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1410045054, data2: 35648, data3: 17852, data4: [168, 162, 106, 11, 137, 76, 189, 162] }, pid: 15u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_FirmwareDate: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1410045054, data2: 35648, data3: 17852, data4: [168, 162, 106, 11, 137, 76, 189, 162] }, pid: 17u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_FirmwareRevision: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1410045054, data2: 35648, data3: 17852, data4: [168, 162, 106, 11, 137, 76, 189, 162] }, pid: 19u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_FirmwareVersion: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1410045054, data2: 35648, data3: 17852, data4: [168, 162, 106, 11, 137, 76, 189, 162] }, pid: 18u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_FirstInstallDate: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2212127526, data2: 38822, data3: 16520, data4: [148, 83, 161, 146, 63, 87, 59, 41] }, pid: 101u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_FriendlyName: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 14u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_FriendlyNameAttributes: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2161647270, data2: 29811, data3: 19212, data4: [130, 22, 239, 193, 26, 44, 76, 139] }, pid: 3u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_GenericDriverInstalled: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2830656989, data2: 11837, data3: 16532, data4: [173, 151, 229, 147, 167, 12, 117, 214] }, pid: 18u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_HardwareIds: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 3u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_HasProblem: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1410045054, data2: 35648, data3: 17852, data4: [168, 162, 106, 11, 137, 76, 189, 162] }, pid: 6u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_InLocalMachineContainer: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2357121542, data2: 16266, data3: 18471, data4: [179, 171, 174, 158, 31, 174, 252, 108] }, pid: 4u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_InstallDate: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2212127526, data2: 38822, data3: 16520, data4: [148, 83, 161, 146, 63, 87, 59, 41] }, pid: 100u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_InstallState: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 36u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_InstanceId: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 256u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_IsAssociateableByUserAction: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2161647270, data2: 29811, data3: 19212, data4: [130, 22, 239, 193, 26, 44, 76, 139] }, pid: 7u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_IsPresent: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1410045054, data2: 35648, data3: 17852, data4: [168, 162, 106, 11, 137, 76, 189, 162] }, pid: 5u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_IsRebootRequired: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1410045054, data2: 35648, data3: 17852, data4: [168, 162, 106, 11, 137, 76, 189, 162] }, pid: 16u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_LastArrivalDate: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2212127526, data2: 38822, data3: 16520, data4: [148, 83, 161, 146, 63, 87, 59, 41] }, pid: 102u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_LastRemovalDate: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2212127526, data2: 38822, data3: 16520, data4: [148, 83, 161, 146, 63, 87, 59, 41] }, pid: 103u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_Legacy: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2152296704, data2: 35955, data3: 18617, data4: [170, 217, 206, 56, 126, 25, 197, 110] }, pid: 3u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_LegacyBusType: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 22u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_LocationInfo: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 15u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_LocationPaths: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 37u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_LowerFilters: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 20u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_Manufacturer: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 13u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_ManufacturerAttributes: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2161647270, data2: 29811, data3: 19212, data4: [130, 22, 239, 193, 26, 44, 76, 139] }, pid: 4u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_MatchingDeviceId: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2830656989, data2: 11837, data3: 16532, data4: [173, 151, 229, 147, 167, 12, 117, 214] }, pid: 8u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_Model: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2026065864, data2: 4170, data3: 19146, data4: [158, 164, 82, 77, 82, 153, 110, 87] }, pid: 39u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_ModelId: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2161647270, data2: 29811, data3: 19212, data4: [130, 22, 239, 193, 26, 44, 76, 139] }, pid: 2u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_NoConnectSound: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2830656989, data2: 11837, data3: 16532, data4: [173, 151, 229, 147, 167, 12, 117, 214] }, pid: 17u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_Numa_Node: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1410045054, data2: 35648, data3: 17852, data4: [168, 162, 106, 11, 137, 76, 189, 162] }, pid: 3u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_Numa_Proximity_Domain: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1410045054, data2: 35648, data3: 17852, data4: [168, 162, 106, 11, 137, 76, 189, 162] }, pid: 1u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_PDOName: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 16u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_Parent: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1128310469, data2: 37882, data3: 18182, data4: [151, 44, 123, 100, 128, 8, 165, 167] }, pid: 8u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_PhysicalDeviceLocation: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1410045054, data2: 35648, data3: 17852, data4: [168, 162, 106, 11, 137, 76, 189, 162] }, pid: 9u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_PostInstallInProgress: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1410045054, data2: 35648, data3: 17852, data4: [168, 162, 106, 11, 137, 76, 189, 162] }, pid: 13u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_PowerData: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 32u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_PowerRelations: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1128310469, data2: 37882, data3: 18182, data4: [151, 44, 123, 100, 128, 8, 165, 167] }, pid: 6u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_PresenceNotForDevice: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2161647270, data2: 29811, data3: 19212, data4: [130, 22, 239, 193, 26, 44, 76, 139] }, pid: 5u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_ProblemCode: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1128310469, data2: 37882, data3: 18182, data4: [151, 44, 123, 100, 128, 8, 165, 167] }, pid: 3u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_ProblemStatus: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1128310469, data2: 37882, data3: 18182, data4: [151, 44, 123, 100, 128, 8, 165, 167] }, pid: 12u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_RemovalPolicy: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 33u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_RemovalPolicyDefault: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 34u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_RemovalPolicyOverride: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 35u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_RemovalRelations: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1128310469, data2: 37882, data3: 18182, data4: [151, 44, 123, 100, 128, 8, 165, 167] }, pid: 5u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_Reported: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2152296704, data2: 35955, data3: 18617, data4: [170, 217, 206, 56, 126, 25, 197, 110] }, pid: 2u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_ReportedDeviceIdsHash: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1410045054, data2: 35648, data3: 17852, data4: [168, 162, 106, 11, 137, 76, 189, 162] }, pid: 8u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_ResourcePickerExceptions: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2830656989, data2: 11837, data3: 16532, data4: [173, 151, 229, 147, 167, 12, 117, 214] }, pid: 13u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_ResourcePickerTags: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2830656989, data2: 11837, data3: 16532, data4: [173, 151, 229, 147, 167, 12, 117, 214] }, pid: 12u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_SafeRemovalRequired: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2950264384, data2: 34467, data3: 16912, data4: [182, 124, 40, 156, 65, 170, 190, 85] }, pid: 2u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_SafeRemovalRequiredOverride: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2950264384, data2: 34467, data3: 16912, data4: [182, 124, 40, 156, 65, 170, 190, 85] }, pid: 3u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_Security: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 25u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_SecuritySDS: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 26u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_Service: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 6u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_SessionId: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2212127526, data2: 38822, data3: 16520, data4: [148, 83, 161, 146, 63, 87, 59, 41] }, pid: 6u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_ShowInUninstallUI: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2161647270, data2: 29811, data3: 19212, data4: [130, 22, 239, 193, 26, 44, 76, 139] }, pid: 8u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_Siblings: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1128310469, data2: 37882, data3: 18182, data4: [151, 44, 123, 100, 128, 8, 165, 167] }, pid: 10u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_SignalStrength: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2161647270, data2: 29811, data3: 19212, data4: [130, 22, 239, 193, 26, 44, 76, 139] }, pid: 6u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_SoftRestartSupported: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1410045054, data2: 35648, data3: 17852, data4: [168, 162, 106, 11, 137, 76, 189, 162] }, pid: 22u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_Stack: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1410045054, data2: 35648, data3: 17852, data4: [168, 162, 106, 11, 137, 76, 189, 162] }, pid: 14u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_TransportRelations: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 1128310469, data2: 37882, data3: 18182, data4: [151, 44, 123, 100, 128, 8, 165, 167] }, pid: 11u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_UINumber: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 18u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_UINumberDescFormat: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 31u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_Device_UpperFilters: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 2757502286, data2: 57116, data3: 20221, data4: [128, 32, 103, 209, 70, 168, 80, 224] }, pid: 19u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DrvPkg_BrandingIcon: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 3480468305, data2: 15039, data3: 17570, data4: [133, 224, 154, 61, 199, 161, 33, 50] }, pid: 7u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DrvPkg_DetailedDescription: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 3480468305, data2: 15039, data3: 17570, data4: [133, 224, 154, 61, 199, 161, 33, 50] }, pid: 4u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DrvPkg_DocumentationLink: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 3480468305, data2: 15039, data3: 17570, data4: [133, 224, 154, 61, 199, 161, 33, 50] }, pid: 5u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DrvPkg_Icon: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 3480468305, data2: 15039, data3: 17570, data4: [133, 224, 154, 61, 199, 161, 33, 50] }, pid: 6u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DrvPkg_Model: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 3480468305, data2: 15039, data3: 17570, data4: [133, 224, 154, 61, 199, 161, 33, 50] }, pid: 2u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_DrvPkg_VendorWebSite: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 3480468305, data2: 15039, data3: 17570, data4: [133, 224, 154, 61, 199, 161, 33, 50] }, pid: 3u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPKEY_NAME: DEVPROPKEY = DEVPROPKEY { fmtid: ::windows_sys::core::GUID { data1: 3072717104, data2: 18415, data3: 4122, data4: [165, 241, 2, 96, 140, 158, 235, 172] }, pid: 10u32 };
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPROPID_FIRST_USABLE: u32 = 2u32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPROP_MASK_TYPE: u32 = 4095u32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPROP_MASK_TYPEMOD: u32 = 61440u32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPROP_TYPEMOD_ARRAY: u32 = 4096u32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPROP_TYPEMOD_LIST: u32 = 8192u32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPROP_TYPE_BOOLEAN: u32 = 17u32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPROP_TYPE_BYTE: u32 = 3u32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPROP_TYPE_CURRENCY: u32 = 14u32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPROP_TYPE_DATE: u32 = 15u32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPROP_TYPE_DECIMAL: u32 = 12u32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPROP_TYPE_DEVPROPKEY: u32 = 21u32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPROP_TYPE_DEVPROPTYPE: u32 = 22u32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPROP_TYPE_DOUBLE: u32 = 11u32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPROP_TYPE_EMPTY: u32 = 0u32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPROP_TYPE_ERROR: u32 = 23u32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPROP_TYPE_FILETIME: u32 = 16u32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPROP_TYPE_FLOAT: u32 = 10u32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPROP_TYPE_GUID: u32 = 13u32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPROP_TYPE_INT16: u32 = 4u32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPROP_TYPE_INT32: u32 = 6u32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPROP_TYPE_INT64: u32 = 8u32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPROP_TYPE_NTSTATUS: u32 = 24u32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPROP_TYPE_NULL: u32 = 1u32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPROP_TYPE_SBYTE: u32 = 2u32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPROP_TYPE_SECURITY_DESCRIPTOR: u32 = 19u32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPROP_TYPE_SECURITY_DESCRIPTOR_STRING: u32 = 20u32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPROP_TYPE_STRING: u32 = 18u32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPROP_TYPE_STRING_INDIRECT: u32 = 25u32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPROP_TYPE_UINT16: u32 = 5u32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPROP_TYPE_UINT32: u32 = 7u32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPROP_TYPE_UINT64: u32 = 9u32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const MAX_DEVPROP_TYPE: u32 = 25u32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const MAX_DEVPROP_TYPEMOD: u32 = 8192u32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub type DEVPROPSTORE = i32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPROP_STORE_SYSTEM: DEVPROPSTORE = 0i32;
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub const DEVPROP_STORE_USER: DEVPROPSTORE = 1i32;
#[repr(C)]
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub struct DEVPROPCOMPKEY {
    pub Key: DEVPROPKEY,
    pub Store: DEVPROPSTORE,
    pub LocaleName: ::windows_sys::core::PCWSTR,
}
impl ::core::marker::Copy for DEVPROPCOMPKEY {}
impl ::core::clone::Clone for DEVPROPCOMPKEY {
    fn clone(&self) -> Self {
        *self
    }
}
#[repr(C)]
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub struct DEVPROPERTY {
    pub CompKey: DEVPROPCOMPKEY,
    pub Type: u32,
    pub BufferSize: u32,
    pub Buffer: *mut ::core::ffi::c_void,
}
impl ::core::marker::Copy for DEVPROPERTY {}
impl ::core::clone::Clone for DEVPROPERTY {
    fn clone(&self) -> Self {
        *self
    }
}
#[repr(C)]
#[doc = "*Required features: `\"Win32_Devices_Properties\"`*"]
pub struct DEVPROPKEY {
    pub fmtid: ::windows_sys::core::GUID,
    pub pid: u32,
}
impl ::core::marker::Copy for DEVPROPKEY {}
impl ::core::clone::Clone for DEVPROPKEY {
    fn clone(&self) -> Self {
        *self
    }
}
