pub type IWPCGamesSettings = *mut ::core::ffi::c_void;
pub type IWPCProviderConfig = *mut ::core::ffi::c_void;
pub type IWPCProviderState = *mut ::core::ffi::c_void;
pub type IWPCProviderSupport = *mut ::core::ffi::c_void;
pub type IWPCSettings = *mut ::core::ffi::c_void;
pub type IWPCWebSettings = *mut ::core::ffi::c_void;
pub type IWindowsParentalControls = *mut ::core::ffi::c_void;
pub type IWindowsParentalControlsCore = *mut ::core::ffi::c_void;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const ARRAY_SEP_CHAR: u32 = 9u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const FACILITY_WPC: u32 = 2457u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Event_AppBlocked: i32 = -1342177264i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Event_AppOverride: i32 = -1342177263i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Event_Application: i32 = -1342177260i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Event_ComputerUsage: i32 = -1342177259i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Event_ContentUsage: i32 = -1342177258i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Event_Custom: i32 = -1342177267i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Event_EmailContact: i32 = -1342177266i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Event_EmailReceived: i32 = -1342177276i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Event_EmailSent: i32 = -1342177275i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Event_FileDownload: i32 = -1342177270i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Event_GameStart: i32 = -1342177278i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Event_IMContact: i32 = -1342177265i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Event_IMFeature: i32 = -1342177269i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Event_IMInvitation: i32 = -1342177273i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Event_IMJoin: i32 = -1342177272i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Event_IMLeave: i32 = -1342177271i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Event_MediaPlayback: i32 = -1342177274i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Event_SettingChange: i32 = -1342177279i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Event_UrlVisit: i32 = -1342177277i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Event_WebOverride: i32 = -1342177262i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Event_WebsiteVisit: i32 = -1342177261i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Keyword_ThirdParty: i32 = 268435462i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Keyword_WPC: i32 = 268435461i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Opcode_Launch: i32 = 805306390i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Opcode_Locate: i32 = 805306388i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Opcode_Modify: i32 = 805306389i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Opcode_System: i32 = 805306391i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Opcode_Web: i32 = 805306392i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Publisher_Name: i32 = -1879048191i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Task_AppBlocked: i32 = 1879048208i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Task_AppOverride: i32 = 1879048209i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Task_Application: i32 = 1879048212i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Task_ComputerUsage: i32 = 1879048213i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Task_ContentUsage: i32 = 1879048214i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Task_Custom: i32 = 1879048205i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Task_EmailContact: i32 = 1879048206i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Task_EmailReceived: i32 = 1879048196i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Task_EmailSent: i32 = 1879048197i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Task_FileDownload: i32 = 1879048202i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Task_GameStart: i32 = 1879048194i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Task_IMContact: i32 = 1879048207i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Task_IMFeature: i32 = 1879048203i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Task_IMInvitation: i32 = 1879048199i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Task_IMJoin: i32 = 1879048200i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Task_IMLeave: i32 = 1879048201i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Task_MediaPlayback: i32 = 1879048198i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Task_SettingChange: i32 = 1879048193i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Task_UrlVisit: i32 = 1879048195i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Task_WebOverride: i32 = 1879048210i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const MSG_Task_WebsiteVisit: i32 = 1879048211i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCCHANNEL: u32 = 16u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCEVENT_APPLICATION_value: u32 = 20u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCEVENT_APPOVERRIDE_value: u32 = 17u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCEVENT_COMPUTERUSAGE_value: u32 = 21u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCEVENT_CONTENTUSAGE_value: u32 = 22u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCEVENT_CUSTOM_value: u32 = 13u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCEVENT_EMAIL_CONTACT_value: u32 = 14u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCEVENT_EMAIL_RECEIVED_value: u32 = 4u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCEVENT_EMAIL_SENT_value: u32 = 5u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCEVENT_GAME_START_value: u32 = 2u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCEVENT_IM_CONTACT_value: u32 = 15u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCEVENT_IM_FEATURE_value: u32 = 11u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCEVENT_IM_INVITATION_value: u32 = 7u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCEVENT_IM_JOIN_value: u32 = 8u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCEVENT_IM_LEAVE_value: u32 = 9u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCEVENT_MEDIA_PLAYBACK_value: u32 = 6u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCEVENT_SYSTEM_APPBLOCKED_value: u32 = 16u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCEVENT_SYS_SETTINGCHANGE_value: u32 = 1u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCEVENT_WEBOVERRIDE_value: u32 = 18u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCEVENT_WEB_FILEDOWNLOAD_value: u32 = 10u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCEVENT_WEB_URLVISIT_value: u32 = 3u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCEVENT_WEB_WEBSITEVISIT_value: u32 = 19u32;
pub const WPCPROV: ::windows_sys::core::GUID = ::windows_sys::core::GUID { data1: 17367141, data2: 46183, data3: 17667, data4: [155, 40, 83, 55, 102, 118, 16, 135] };
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCPROV_KEYWORD_ThirdParty: u32 = 32u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCPROV_KEYWORD_WPC: u32 = 16u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCPROV_TASK_AppBlocked: u32 = 16u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCPROV_TASK_AppOverride: u32 = 17u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCPROV_TASK_Application: u32 = 20u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCPROV_TASK_ComputerUsage: u32 = 21u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCPROV_TASK_ContentUsage: u32 = 22u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCPROV_TASK_Custom: u32 = 13u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCPROV_TASK_EmailContact: u32 = 14u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCPROV_TASK_EmailReceived: u32 = 4u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCPROV_TASK_EmailSent: u32 = 5u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCPROV_TASK_FileDownload: u32 = 10u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCPROV_TASK_GameStart: u32 = 2u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCPROV_TASK_IMContact: u32 = 15u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCPROV_TASK_IMFeature: u32 = 11u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCPROV_TASK_IMInvitation: u32 = 7u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCPROV_TASK_IMJoin: u32 = 8u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCPROV_TASK_IMLeave: u32 = 9u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCPROV_TASK_MediaPlayback: u32 = 6u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCPROV_TASK_SettingChange: u32 = 1u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCPROV_TASK_UrlVisit: u32 = 3u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCPROV_TASK_WebOverride: u32 = 18u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCPROV_TASK_WebsiteVisit: u32 = 19u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_APP_LAUNCH: u32 = 22u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_LOCATE: u32 = 20u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_MODIFY: u32 = 21u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SYSTEM: u32 = 23u32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_WEB: u32 = 24u32;
pub const WindowsParentalControls: ::windows_sys::core::GUID = ::windows_sys::core::GUID { data1: 3883714715, data2: 29697, data3: 19460, data4: [140, 237, 20, 157, 179, 90, 221, 4] };
pub const WpcProviderSupport: ::windows_sys::core::GUID = ::windows_sys::core::GUID { data1: 3138963360, data2: 8582, data3: 19424, data4: [151, 216, 4, 132, 123, 98, 142, 2] };
pub const WpcSettingsProvider: ::windows_sys::core::GUID = ::windows_sys::core::GUID { data1: 895352746, data2: 15263, data3: 17244, data4: [180, 40, 93, 68, 41, 11, 197, 242] };
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub type WPCFLAG_IM_FEATURE = i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_IM_FEATURE_NONE: WPCFLAG_IM_FEATURE = 0i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_IM_FEATURE_VIDEO: WPCFLAG_IM_FEATURE = 1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_IM_FEATURE_AUDIO: WPCFLAG_IM_FEATURE = 2i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_IM_FEATURE_GAME: WPCFLAG_IM_FEATURE = 4i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_IM_FEATURE_SMS: WPCFLAG_IM_FEATURE = 8i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_IM_FEATURE_FILESWAP: WPCFLAG_IM_FEATURE = 16i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_IM_FEATURE_URLSWAP: WPCFLAG_IM_FEATURE = 32i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_IM_FEATURE_SENDING: WPCFLAG_IM_FEATURE = -2147483648i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_IM_FEATURE_ALL: WPCFLAG_IM_FEATURE = -1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub type WPCFLAG_IM_LEAVE = i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_IM_LEAVE_NORMAL: WPCFLAG_IM_LEAVE = 0i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_IM_LEAVE_FORCED: WPCFLAG_IM_LEAVE = 1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_IM_LEAVE_CONVERSATION_END: WPCFLAG_IM_LEAVE = 2i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub type WPCFLAG_ISBLOCKED = i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_ISBLOCKED_NOTBLOCKED: WPCFLAG_ISBLOCKED = 0i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_ISBLOCKED_IMBLOCKED: WPCFLAG_ISBLOCKED = 1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_ISBLOCKED_EMAILBLOCKED: WPCFLAG_ISBLOCKED = 2i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_ISBLOCKED_MEDIAPLAYBACKBLOCKED: WPCFLAG_ISBLOCKED = 4i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_ISBLOCKED_WEBBLOCKED: WPCFLAG_ISBLOCKED = 8i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_ISBLOCKED_GAMESBLOCKED: WPCFLAG_ISBLOCKED = 16i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_ISBLOCKED_CONTACTBLOCKED: WPCFLAG_ISBLOCKED = 32i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_ISBLOCKED_FEATUREBLOCKED: WPCFLAG_ISBLOCKED = 64i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_ISBLOCKED_DOWNLOADBLOCKED: WPCFLAG_ISBLOCKED = 128i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_ISBLOCKED_RATINGBLOCKED: WPCFLAG_ISBLOCKED = 256i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_ISBLOCKED_DESCRIPTORBLOCKED: WPCFLAG_ISBLOCKED = 512i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_ISBLOCKED_EXPLICITBLOCK: WPCFLAG_ISBLOCKED = 1024i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_ISBLOCKED_BADPASS: WPCFLAG_ISBLOCKED = 2048i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_ISBLOCKED_MAXHOURS: WPCFLAG_ISBLOCKED = 4096i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_ISBLOCKED_SPECHOURS: WPCFLAG_ISBLOCKED = 8192i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_ISBLOCKED_SETTINGSCHANGEBLOCKED: WPCFLAG_ISBLOCKED = 16384i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_ISBLOCKED_ATTACHMENTBLOCKED: WPCFLAG_ISBLOCKED = 32768i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_ISBLOCKED_SENDERBLOCKED: WPCFLAG_ISBLOCKED = 65536i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_ISBLOCKED_RECEIVERBLOCKED: WPCFLAG_ISBLOCKED = 131072i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_ISBLOCKED_NOTEXPLICITLYALLOWED: WPCFLAG_ISBLOCKED = 262144i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_ISBLOCKED_NOTINLIST: WPCFLAG_ISBLOCKED = 524288i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_ISBLOCKED_CATEGORYBLOCKED: WPCFLAG_ISBLOCKED = 1048576i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_ISBLOCKED_CATEGORYNOTINLIST: WPCFLAG_ISBLOCKED = 2097152i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_ISBLOCKED_NOTKIDS: WPCFLAG_ISBLOCKED = 4194304i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_ISBLOCKED_UNRATED: WPCFLAG_ISBLOCKED = 8388608i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_ISBLOCKED_NOACCESS: WPCFLAG_ISBLOCKED = 16777216i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_ISBLOCKED_INTERNALERROR: WPCFLAG_ISBLOCKED = -1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub type WPCFLAG_LOGOFF_TYPE = i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_LOGOFF_TYPE_LOGOUT: WPCFLAG_LOGOFF_TYPE = 0i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_LOGOFF_TYPE_RESTART: WPCFLAG_LOGOFF_TYPE = 1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_LOGOFF_TYPE_SHUTDOWN: WPCFLAG_LOGOFF_TYPE = 2i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_LOGOFF_TYPE_FUS: WPCFLAG_LOGOFF_TYPE = 4i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_LOGOFF_TYPE_FORCEDFUS: WPCFLAG_LOGOFF_TYPE = 8i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub type WPCFLAG_OVERRIDE = i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_APPLICATION: WPCFLAG_OVERRIDE = 1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub type WPCFLAG_RESTRICTION = i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_NO_RESTRICTION: WPCFLAG_RESTRICTION = 0i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_LOGGING_REQUIRED: WPCFLAG_RESTRICTION = 1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_WEB_FILTERED: WPCFLAG_RESTRICTION = 2i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_HOURS_RESTRICTED: WPCFLAG_RESTRICTION = 4i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_GAMES_BLOCKED: WPCFLAG_RESTRICTION = 8i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_APPS_RESTRICTED: WPCFLAG_RESTRICTION = 16i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_TIME_ALLOWANCE_RESTRICTED: WPCFLAG_RESTRICTION = 32i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_GAMES_RESTRICTED: WPCFLAG_RESTRICTION = 64i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub type WPCFLAG_VISIBILITY = i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_WPC_VISIBLE: WPCFLAG_VISIBILITY = 0i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_WPC_HIDDEN: WPCFLAG_VISIBILITY = 1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub type WPCFLAG_WEB_SETTING = i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_WEB_SETTING_NOTBLOCKED: WPCFLAG_WEB_SETTING = 0i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPCFLAG_WEB_SETTING_DOWNLOADSBLOCKED: WPCFLAG_WEB_SETTING = 1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub type WPC_ARGS_APPLICATIONEVENT = i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_APPLICATIONEVENT_SERIALIZEDAPPLICATION: WPC_ARGS_APPLICATIONEVENT = 0i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_APPLICATIONEVENT_DECISION: WPC_ARGS_APPLICATIONEVENT = 1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_APPLICATIONEVENT_PROCESSID: WPC_ARGS_APPLICATIONEVENT = 2i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_APPLICATIONEVENT_CREATIONTIME: WPC_ARGS_APPLICATIONEVENT = 3i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_APPLICATIONEVENT_TIMEUSED: WPC_ARGS_APPLICATIONEVENT = 4i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_APPLICATIONEVENT_CARGS: WPC_ARGS_APPLICATIONEVENT = 5i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub type WPC_ARGS_APPOVERRIDEEVENT = i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_APPOVERRIDEEVENT_USERID: WPC_ARGS_APPOVERRIDEEVENT = 0i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_APPOVERRIDEEVENT_PATH: WPC_ARGS_APPOVERRIDEEVENT = 1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_APPOVERRIDEEVENT_REASON: WPC_ARGS_APPOVERRIDEEVENT = 2i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_APPOVERRIDEEVENT_CARGS: WPC_ARGS_APPOVERRIDEEVENT = 3i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub type WPC_ARGS_COMPUTERUSAGEEVENT = i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_COMPUTERUSAGEEVENT_ID: WPC_ARGS_COMPUTERUSAGEEVENT = 0i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_COMPUTERUSAGEEVENT_TIMEUSED: WPC_ARGS_COMPUTERUSAGEEVENT = 1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_COMPUTERUSAGEEVENT_CARGS: WPC_ARGS_COMPUTERUSAGEEVENT = 2i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub type WPC_ARGS_CONTENTUSAGEEVENT = i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONTENTUSAGEEVENT_CONTENTPROVIDERID: WPC_ARGS_CONTENTUSAGEEVENT = 0i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONTENTUSAGEEVENT_CONTENTPROVIDERTITLE: WPC_ARGS_CONTENTUSAGEEVENT = 1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONTENTUSAGEEVENT_ID: WPC_ARGS_CONTENTUSAGEEVENT = 2i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONTENTUSAGEEVENT_TITLE: WPC_ARGS_CONTENTUSAGEEVENT = 3i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONTENTUSAGEEVENT_CATEGORY: WPC_ARGS_CONTENTUSAGEEVENT = 4i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONTENTUSAGEEVENT_RATINGS: WPC_ARGS_CONTENTUSAGEEVENT = 5i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONTENTUSAGEEVENT_DECISION: WPC_ARGS_CONTENTUSAGEEVENT = 6i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONTENTUSAGEEVENT_CARGS: WPC_ARGS_CONTENTUSAGEEVENT = 7i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub type WPC_ARGS_CONVERSATIONINITEVENT = i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONVERSATIONINITEVENT_APPNAME: WPC_ARGS_CONVERSATIONINITEVENT = 0i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONVERSATIONINITEVENT_APPVERSION: WPC_ARGS_CONVERSATIONINITEVENT = 1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONVERSATIONINITEVENT_ACCOUNTNAME: WPC_ARGS_CONVERSATIONINITEVENT = 2i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONVERSATIONINITEVENT_CONVID: WPC_ARGS_CONVERSATIONINITEVENT = 3i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONVERSATIONINITEVENT_REQUESTINGIP: WPC_ARGS_CONVERSATIONINITEVENT = 4i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONVERSATIONINITEVENT_SENDER: WPC_ARGS_CONVERSATIONINITEVENT = 5i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONVERSATIONINITEVENT_REASON: WPC_ARGS_CONVERSATIONINITEVENT = 6i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONVERSATIONINITEVENT_RECIPCOUNT: WPC_ARGS_CONVERSATIONINITEVENT = 7i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONVERSATIONINITEVENT_RECIPIENT: WPC_ARGS_CONVERSATIONINITEVENT = 8i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONVERSATIONINITEVENT_CARGS: WPC_ARGS_CONVERSATIONINITEVENT = 9i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub type WPC_ARGS_CONVERSATIONJOINEVENT = i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONVERSATIONJOINEVENT_APPNAME: WPC_ARGS_CONVERSATIONJOINEVENT = 0i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONVERSATIONJOINEVENT_APPVERSION: WPC_ARGS_CONVERSATIONJOINEVENT = 1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONVERSATIONJOINEVENT_ACCOUNTNAME: WPC_ARGS_CONVERSATIONJOINEVENT = 2i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONVERSATIONJOINEVENT_CONVID: WPC_ARGS_CONVERSATIONJOINEVENT = 3i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONVERSATIONJOINEVENT_JOININGIP: WPC_ARGS_CONVERSATIONJOINEVENT = 4i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONVERSATIONJOINEVENT_JOININGUSER: WPC_ARGS_CONVERSATIONJOINEVENT = 5i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONVERSATIONJOINEVENT_REASON: WPC_ARGS_CONVERSATIONJOINEVENT = 6i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONVERSATIONJOINEVENT_MEMBERCOUNT: WPC_ARGS_CONVERSATIONJOINEVENT = 7i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONVERSATIONJOINEVENT_MEMBER: WPC_ARGS_CONVERSATIONJOINEVENT = 8i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONVERSATIONJOINEVENT_SENDER: WPC_ARGS_CONVERSATIONJOINEVENT = 9i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONVERSATIONJOINEVENT_CARGS: WPC_ARGS_CONVERSATIONJOINEVENT = 10i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub type WPC_ARGS_CONVERSATIONLEAVEEVENT = i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONVERSATIONLEAVEEVENT_APPNAME: WPC_ARGS_CONVERSATIONLEAVEEVENT = 0i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONVERSATIONLEAVEEVENT_APPVERSION: WPC_ARGS_CONVERSATIONLEAVEEVENT = 1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONVERSATIONLEAVEEVENT_ACCOUNTNAME: WPC_ARGS_CONVERSATIONLEAVEEVENT = 2i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONVERSATIONLEAVEEVENT_CONVID: WPC_ARGS_CONVERSATIONLEAVEEVENT = 3i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONVERSATIONLEAVEEVENT_LEAVINGIP: WPC_ARGS_CONVERSATIONLEAVEEVENT = 4i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONVERSATIONLEAVEEVENT_LEAVINGUSER: WPC_ARGS_CONVERSATIONLEAVEEVENT = 5i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONVERSATIONLEAVEEVENT_REASON: WPC_ARGS_CONVERSATIONLEAVEEVENT = 6i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONVERSATIONLEAVEEVENT_MEMBERCOUNT: WPC_ARGS_CONVERSATIONLEAVEEVENT = 7i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONVERSATIONLEAVEEVENT_MEMBER: WPC_ARGS_CONVERSATIONLEAVEEVENT = 8i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONVERSATIONLEAVEEVENT_FLAGS: WPC_ARGS_CONVERSATIONLEAVEEVENT = 9i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CONVERSATIONLEAVEEVENT_CARGS: WPC_ARGS_CONVERSATIONLEAVEEVENT = 10i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub type WPC_ARGS_CUSTOMEVENT = i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CUSTOMEVENT_PUBLISHER: WPC_ARGS_CUSTOMEVENT = 0i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CUSTOMEVENT_APPNAME: WPC_ARGS_CUSTOMEVENT = 1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CUSTOMEVENT_APPVERSION: WPC_ARGS_CUSTOMEVENT = 2i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CUSTOMEVENT_EVENT: WPC_ARGS_CUSTOMEVENT = 3i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CUSTOMEVENT_VALUE1: WPC_ARGS_CUSTOMEVENT = 4i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CUSTOMEVENT_VALUE2: WPC_ARGS_CUSTOMEVENT = 5i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CUSTOMEVENT_VALUE3: WPC_ARGS_CUSTOMEVENT = 6i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CUSTOMEVENT_BLOCKED: WPC_ARGS_CUSTOMEVENT = 7i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CUSTOMEVENT_REASON: WPC_ARGS_CUSTOMEVENT = 8i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_CUSTOMEVENT_CARGS: WPC_ARGS_CUSTOMEVENT = 9i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub type WPC_ARGS_EMAILCONTACTEVENT = i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_EMAILCONTACTEVENT_APPNAME: WPC_ARGS_EMAILCONTACTEVENT = 0i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_EMAILCONTACTEVENT_APPVERSION: WPC_ARGS_EMAILCONTACTEVENT = 1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_EMAILCONTACTEVENT_OLDNAME: WPC_ARGS_EMAILCONTACTEVENT = 2i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_EMAILCONTACTEVENT_OLDID: WPC_ARGS_EMAILCONTACTEVENT = 3i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_EMAILCONTACTEVENT_NEWNAME: WPC_ARGS_EMAILCONTACTEVENT = 4i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_EMAILCONTACTEVENT_NEWID: WPC_ARGS_EMAILCONTACTEVENT = 5i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_EMAILCONTACTEVENT_REASON: WPC_ARGS_EMAILCONTACTEVENT = 6i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_EMAILCONTACTEVENT_EMAILACCOUNT: WPC_ARGS_EMAILCONTACTEVENT = 7i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_EMAILCONTACTEVENT_CARGS: WPC_ARGS_EMAILCONTACTEVENT = 8i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub type WPC_ARGS_EMAILRECEIEVEDEVENT = i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_EMAILRECEIEVEDEVENT_SENDER: WPC_ARGS_EMAILRECEIEVEDEVENT = 0i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_EMAILRECEIEVEDEVENT_APPNAME: WPC_ARGS_EMAILRECEIEVEDEVENT = 1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_EMAILRECEIEVEDEVENT_APPVERSION: WPC_ARGS_EMAILRECEIEVEDEVENT = 2i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_EMAILRECEIEVEDEVENT_SUBJECT: WPC_ARGS_EMAILRECEIEVEDEVENT = 3i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_EMAILRECEIEVEDEVENT_REASON: WPC_ARGS_EMAILRECEIEVEDEVENT = 4i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_EMAILRECEIEVEDEVENT_RECIPCOUNT: WPC_ARGS_EMAILRECEIEVEDEVENT = 5i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_EMAILRECEIEVEDEVENT_RECIPIENT: WPC_ARGS_EMAILRECEIEVEDEVENT = 6i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_EMAILRECEIEVEDEVENT_ATTACHCOUNT: WPC_ARGS_EMAILRECEIEVEDEVENT = 7i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_EMAILRECEIEVEDEVENT_ATTACHMENTNAME: WPC_ARGS_EMAILRECEIEVEDEVENT = 8i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_EMAILRECEIEVEDEVENT_RECEIVEDTIME: WPC_ARGS_EMAILRECEIEVEDEVENT = 9i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_EMAILRECEIEVEDEVENT_EMAILACCOUNT: WPC_ARGS_EMAILRECEIEVEDEVENT = 10i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_EMAILRECEIEVEDEVENT_CARGS: WPC_ARGS_EMAILRECEIEVEDEVENT = 11i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub type WPC_ARGS_EMAILSENTEVENT = i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_EMAILSENTEVENT_SENDER: WPC_ARGS_EMAILSENTEVENT = 0i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_EMAILSENTEVENT_APPNAME: WPC_ARGS_EMAILSENTEVENT = 1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_EMAILSENTEVENT_APPVERSION: WPC_ARGS_EMAILSENTEVENT = 2i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_EMAILSENTEVENT_SUBJECT: WPC_ARGS_EMAILSENTEVENT = 3i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_EMAILSENTEVENT_REASON: WPC_ARGS_EMAILSENTEVENT = 4i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_EMAILSENTEVENT_RECIPCOUNT: WPC_ARGS_EMAILSENTEVENT = 5i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_EMAILSENTEVENT_RECIPIENT: WPC_ARGS_EMAILSENTEVENT = 6i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_EMAILSENTEVENT_ATTACHCOUNT: WPC_ARGS_EMAILSENTEVENT = 7i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_EMAILSENTEVENT_ATTACHMENTNAME: WPC_ARGS_EMAILSENTEVENT = 8i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_EMAILSENTEVENT_EMAILACCOUNT: WPC_ARGS_EMAILSENTEVENT = 9i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_EMAILSENTEVENT_CARGS: WPC_ARGS_EMAILSENTEVENT = 10i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub type WPC_ARGS_FILEDOWNLOADEVENT = i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_FILEDOWNLOADEVENT_URL: WPC_ARGS_FILEDOWNLOADEVENT = 0i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_FILEDOWNLOADEVENT_APPNAME: WPC_ARGS_FILEDOWNLOADEVENT = 1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_FILEDOWNLOADEVENT_VERSION: WPC_ARGS_FILEDOWNLOADEVENT = 2i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_FILEDOWNLOADEVENT_BLOCKED: WPC_ARGS_FILEDOWNLOADEVENT = 3i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_FILEDOWNLOADEVENT_PATH: WPC_ARGS_FILEDOWNLOADEVENT = 4i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_FILEDOWNLOADEVENT_CARGS: WPC_ARGS_FILEDOWNLOADEVENT = 5i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub type WPC_ARGS_GAMESTARTEVENT = i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_GAMESTARTEVENT_APPID: WPC_ARGS_GAMESTARTEVENT = 0i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_GAMESTARTEVENT_INSTANCEID: WPC_ARGS_GAMESTARTEVENT = 1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_GAMESTARTEVENT_APPVERSION: WPC_ARGS_GAMESTARTEVENT = 2i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_GAMESTARTEVENT_PATH: WPC_ARGS_GAMESTARTEVENT = 3i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_GAMESTARTEVENT_RATING: WPC_ARGS_GAMESTARTEVENT = 4i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_GAMESTARTEVENT_RATINGSYSTEM: WPC_ARGS_GAMESTARTEVENT = 5i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_GAMESTARTEVENT_REASON: WPC_ARGS_GAMESTARTEVENT = 6i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_GAMESTARTEVENT_DESCCOUNT: WPC_ARGS_GAMESTARTEVENT = 7i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_GAMESTARTEVENT_DESCRIPTOR: WPC_ARGS_GAMESTARTEVENT = 8i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_GAMESTARTEVENT_PID: WPC_ARGS_GAMESTARTEVENT = 9i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_GAMESTARTEVENT_CARGS: WPC_ARGS_GAMESTARTEVENT = 10i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub type WPC_ARGS_IMCONTACTEVENT = i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_IMCONTACTEVENT_APPNAME: WPC_ARGS_IMCONTACTEVENT = 0i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_IMCONTACTEVENT_APPVERSION: WPC_ARGS_IMCONTACTEVENT = 1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_IMCONTACTEVENT_ACCOUNTNAME: WPC_ARGS_IMCONTACTEVENT = 2i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_IMCONTACTEVENT_OLDNAME: WPC_ARGS_IMCONTACTEVENT = 3i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_IMCONTACTEVENT_OLDID: WPC_ARGS_IMCONTACTEVENT = 4i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_IMCONTACTEVENT_NEWNAME: WPC_ARGS_IMCONTACTEVENT = 5i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_IMCONTACTEVENT_NEWID: WPC_ARGS_IMCONTACTEVENT = 6i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_IMCONTACTEVENT_REASON: WPC_ARGS_IMCONTACTEVENT = 7i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_IMCONTACTEVENT_CARGS: WPC_ARGS_IMCONTACTEVENT = 8i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub type WPC_ARGS_IMFEATUREEVENT = i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_IMFEATUREEVENT_APPNAME: WPC_ARGS_IMFEATUREEVENT = 0i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_IMFEATUREEVENT_APPVERSION: WPC_ARGS_IMFEATUREEVENT = 1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_IMFEATUREEVENT_ACCOUNTNAME: WPC_ARGS_IMFEATUREEVENT = 2i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_IMFEATUREEVENT_CONVID: WPC_ARGS_IMFEATUREEVENT = 3i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_IMFEATUREEVENT_MEDIATYPE: WPC_ARGS_IMFEATUREEVENT = 4i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_IMFEATUREEVENT_REASON: WPC_ARGS_IMFEATUREEVENT = 5i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_IMFEATUREEVENT_RECIPCOUNT: WPC_ARGS_IMFEATUREEVENT = 6i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_IMFEATUREEVENT_RECIPIENT: WPC_ARGS_IMFEATUREEVENT = 7i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_IMFEATUREEVENT_SENDER: WPC_ARGS_IMFEATUREEVENT = 8i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_IMFEATUREEVENT_SENDERIP: WPC_ARGS_IMFEATUREEVENT = 9i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_IMFEATUREEVENT_DATA: WPC_ARGS_IMFEATUREEVENT = 10i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_IMFEATUREEVENT_CARGS: WPC_ARGS_IMFEATUREEVENT = 11i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub type WPC_ARGS_MEDIADOWNLOADEVENT = i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_MEDIADOWNLOADEVENT_APPNAME: WPC_ARGS_MEDIADOWNLOADEVENT = 0i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_MEDIADOWNLOADEVENT_APPVERSION: WPC_ARGS_MEDIADOWNLOADEVENT = 1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_MEDIADOWNLOADEVENT_MEDIATYPE: WPC_ARGS_MEDIADOWNLOADEVENT = 2i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_MEDIADOWNLOADEVENT_PATH: WPC_ARGS_MEDIADOWNLOADEVENT = 3i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_MEDIADOWNLOADEVENT_TITLE: WPC_ARGS_MEDIADOWNLOADEVENT = 4i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_MEDIADOWNLOADEVENT_PML: WPC_ARGS_MEDIADOWNLOADEVENT = 5i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_MEDIADOWNLOADEVENT_ALBUM: WPC_ARGS_MEDIADOWNLOADEVENT = 6i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_MEDIADOWNLOADEVENT_EXPLICIT: WPC_ARGS_MEDIADOWNLOADEVENT = 7i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_MEDIADOWNLOADEVENT_REASON: WPC_ARGS_MEDIADOWNLOADEVENT = 8i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_MEDIADOWNLOADEVENT_CARGS: WPC_ARGS_MEDIADOWNLOADEVENT = 9i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub type WPC_ARGS_MEDIAPLAYBACKEVENT = i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_MEDIAPLAYBACKEVENT_APPNAME: WPC_ARGS_MEDIAPLAYBACKEVENT = 0i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_MEDIAPLAYBACKEVENT_APPVERSION: WPC_ARGS_MEDIAPLAYBACKEVENT = 1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_MEDIAPLAYBACKEVENT_MEDIATYPE: WPC_ARGS_MEDIAPLAYBACKEVENT = 2i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_MEDIAPLAYBACKEVENT_PATH: WPC_ARGS_MEDIAPLAYBACKEVENT = 3i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_MEDIAPLAYBACKEVENT_TITLE: WPC_ARGS_MEDIAPLAYBACKEVENT = 4i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_MEDIAPLAYBACKEVENT_PML: WPC_ARGS_MEDIAPLAYBACKEVENT = 5i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_MEDIAPLAYBACKEVENT_ALBUM: WPC_ARGS_MEDIAPLAYBACKEVENT = 6i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_MEDIAPLAYBACKEVENT_EXPLICIT: WPC_ARGS_MEDIAPLAYBACKEVENT = 7i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_MEDIAPLAYBACKEVENT_REASON: WPC_ARGS_MEDIAPLAYBACKEVENT = 8i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_MEDIAPLAYBACKEVENT_CARGS: WPC_ARGS_MEDIAPLAYBACKEVENT = 9i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub type WPC_ARGS_SAFERAPPBLOCKED = i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_SAFERAPPBLOCKED_TIMESTAMP: WPC_ARGS_SAFERAPPBLOCKED = 0i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_SAFERAPPBLOCKED_USERID: WPC_ARGS_SAFERAPPBLOCKED = 1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_SAFERAPPBLOCKED_PATH: WPC_ARGS_SAFERAPPBLOCKED = 2i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_SAFERAPPBLOCKED_RULEID: WPC_ARGS_SAFERAPPBLOCKED = 3i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_SAFERAPPBLOCKED_CARGS: WPC_ARGS_SAFERAPPBLOCKED = 4i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub type WPC_ARGS_SETTINGSCHANGEEVENT = i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_SETTINGSCHANGEEVENT_CLASS: WPC_ARGS_SETTINGSCHANGEEVENT = 0i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_SETTINGSCHANGEEVENT_SETTING: WPC_ARGS_SETTINGSCHANGEEVENT = 1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_SETTINGSCHANGEEVENT_OWNER: WPC_ARGS_SETTINGSCHANGEEVENT = 2i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_SETTINGSCHANGEEVENT_OLDVAL: WPC_ARGS_SETTINGSCHANGEEVENT = 3i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_SETTINGSCHANGEEVENT_NEWVAL: WPC_ARGS_SETTINGSCHANGEEVENT = 4i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_SETTINGSCHANGEEVENT_REASON: WPC_ARGS_SETTINGSCHANGEEVENT = 5i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_SETTINGSCHANGEEVENT_OPTIONAL: WPC_ARGS_SETTINGSCHANGEEVENT = 6i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_SETTINGSCHANGEEVENT_CARGS: WPC_ARGS_SETTINGSCHANGEEVENT = 7i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub type WPC_ARGS_URLVISITEVENT = i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_URLVISITEVENT_URL: WPC_ARGS_URLVISITEVENT = 0i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_URLVISITEVENT_APPNAME: WPC_ARGS_URLVISITEVENT = 1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_URLVISITEVENT_VERSION: WPC_ARGS_URLVISITEVENT = 2i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_URLVISITEVENT_REASON: WPC_ARGS_URLVISITEVENT = 3i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_URLVISITEVENT_RATINGSYSTEMID: WPC_ARGS_URLVISITEVENT = 4i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_URLVISITEVENT_CATCOUNT: WPC_ARGS_URLVISITEVENT = 5i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_URLVISITEVENT_CATEGORY: WPC_ARGS_URLVISITEVENT = 6i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_URLVISITEVENT_CARGS: WPC_ARGS_URLVISITEVENT = 7i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub type WPC_ARGS_WEBOVERRIDEEVENT = i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_WEBOVERRIDEEVENT_USERID: WPC_ARGS_WEBOVERRIDEEVENT = 0i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_WEBOVERRIDEEVENT_URL: WPC_ARGS_WEBOVERRIDEEVENT = 1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_WEBOVERRIDEEVENT_REASON: WPC_ARGS_WEBOVERRIDEEVENT = 2i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_WEBOVERRIDEEVENT_CARGS: WPC_ARGS_WEBOVERRIDEEVENT = 3i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub type WPC_ARGS_WEBSITEVISITEVENT = i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_WEBSITEVISITEVENT_URL: WPC_ARGS_WEBSITEVISITEVENT = 0i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_WEBSITEVISITEVENT_DECISION: WPC_ARGS_WEBSITEVISITEVENT = 1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_WEBSITEVISITEVENT_CATEGORIES: WPC_ARGS_WEBSITEVISITEVENT = 2i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_WEBSITEVISITEVENT_BLOCKEDCATEGORIES: WPC_ARGS_WEBSITEVISITEVENT = 3i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_WEBSITEVISITEVENT_SERIALIZEDAPPLICATION: WPC_ARGS_WEBSITEVISITEVENT = 4i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_WEBSITEVISITEVENT_TITLE: WPC_ARGS_WEBSITEVISITEVENT = 5i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_WEBSITEVISITEVENT_CONTENTTYPE: WPC_ARGS_WEBSITEVISITEVENT = 6i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_WEBSITEVISITEVENT_REFERRER: WPC_ARGS_WEBSITEVISITEVENT = 7i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_WEBSITEVISITEVENT_TELEMETRY: WPC_ARGS_WEBSITEVISITEVENT = 8i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_ARGS_WEBSITEVISITEVENT_CARGS: WPC_ARGS_WEBSITEVISITEVENT = 9i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub type WPC_MEDIA_EXPLICIT = i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_MEDIA_EXPLICIT_FALSE: WPC_MEDIA_EXPLICIT = 0i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_MEDIA_EXPLICIT_TRUE: WPC_MEDIA_EXPLICIT = 1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_MEDIA_EXPLICIT_UNKNOWN: WPC_MEDIA_EXPLICIT = 2i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub type WPC_MEDIA_TYPE = i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_MEDIA_TYPE_OTHER: WPC_MEDIA_TYPE = 0i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_MEDIA_TYPE_DVD: WPC_MEDIA_TYPE = 1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_MEDIA_TYPE_RECORDED_TV: WPC_MEDIA_TYPE = 2i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_MEDIA_TYPE_AUDIO_FILE: WPC_MEDIA_TYPE = 3i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_MEDIA_TYPE_CD_AUDIO: WPC_MEDIA_TYPE = 4i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_MEDIA_TYPE_VIDEO_FILE: WPC_MEDIA_TYPE = 5i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_MEDIA_TYPE_PICTURE_FILE: WPC_MEDIA_TYPE = 6i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_MEDIA_TYPE_MAX: WPC_MEDIA_TYPE = 7i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub type WPC_SETTINGS = i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_WPC_EXTENSION_PATH: WPC_SETTINGS = 0i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_WPC_EXTENSION_SILO: WPC_SETTINGS = 1i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_WPC_EXTENSION_IMAGE_PATH: WPC_SETTINGS = 2i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_WPC_EXTENSION_DISABLEDIMAGE_PATH: WPC_SETTINGS = 3i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_WPC_EXTENSION_NAME: WPC_SETTINGS = 4i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_WPC_EXTENSION_SUB_TITLE: WPC_SETTINGS = 5i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_SYSTEM_CURRENT_RATING_SYSTEM: WPC_SETTINGS = 6i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_SYSTEM_LAST_LOG_VIEW: WPC_SETTINGS = 7i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_SYSTEM_LOG_VIEW_REMINDER_INTERVAL: WPC_SETTINGS = 8i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_SYSTEM_HTTP_EXEMPTION_LIST: WPC_SETTINGS = 9i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_SYSTEM_URL_EXEMPTION_LIST: WPC_SETTINGS = 10i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_SYSTEM_FILTER_ID: WPC_SETTINGS = 11i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_SYSTEM_FILTER_NAME: WPC_SETTINGS = 12i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_SYSTEM_LOCALE: WPC_SETTINGS = 13i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_ALLOW_BLOCK: WPC_SETTINGS = 14i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_GAME_BLOCKED: WPC_SETTINGS = 15i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_GAME_ALLOW_UNRATED: WPC_SETTINGS = 16i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_GAME_MAX_ALLOWED: WPC_SETTINGS = 17i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_GAME_DENIED_DESCRIPTORS: WPC_SETTINGS = 18i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_USER_WPC_ENABLED: WPC_SETTINGS = 19i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_USER_LOGGING_REQUIRED: WPC_SETTINGS = 20i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_USER_HOURLY_RESTRICTIONS: WPC_SETTINGS = 21i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_USER_OVERRRIDE_REQUESTS: WPC_SETTINGS = 22i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_USER_LOGON_HOURS: WPC_SETTINGS = 23i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_USER_APP_RESTRICTIONS: WPC_SETTINGS = 24i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_WEB_FILTER_ON: WPC_SETTINGS = 25i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_WEB_DOWNLOAD_BLOCKED: WPC_SETTINGS = 26i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_WEB_FILTER_LEVEL: WPC_SETTINGS = 27i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_WEB_BLOCKED_CATEGORY_LIST: WPC_SETTINGS = 28i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_WEB_BLOCK_UNRATED: WPC_SETTINGS = 29i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_WPC_ENABLED: WPC_SETTINGS = 30i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_WPC_LOGGING_REQUIRED: WPC_SETTINGS = 31i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_RATING_SYSTEM_PATH: WPC_SETTINGS = 32i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_WPC_PROVIDER_CURRENT: WPC_SETTINGS = 33i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_USER_TIME_ALLOWANCE: WPC_SETTINGS = 34i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_USER_TIME_ALLOWANCE_RESTRICTIONS: WPC_SETTINGS = 35i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTINGS_GAME_RESTRICTED: WPC_SETTINGS = 36i32;
#[doc = "*Required features: `\"Win32_System_ParentalControls\"`*"]
pub const WPC_SETTING_COUNT: WPC_SETTINGS = 37i32;
