#[cfg(feature = "Win32_Foundation")]
::windows_sys::core::link ! ( "kernel32.dll""system" #[doc = "*Required features: `\"Win32_System_Pipes\"`, `\"Win32_Foundation\"`*"] fn CallNamedPipeA ( lpnamedpipename : :: windows_sys::core::PCSTR , lpinbuffer : *const ::core::ffi::c_void , ninbuffersize : u32 , lpoutbuffer : *mut ::core::ffi::c_void , noutbuffersize : u32 , lpbytesread : *mut u32 , ntimeout : u32 ) -> super::super::Foundation:: BOOL );
#[cfg(feature = "Win32_Foundation")]
::windows_sys::core::link ! ( "kernel32.dll""system" #[doc = "*Required features: `\"Win32_System_Pipes\"`, `\"Win32_Foundation\"`*"] fn CallNamedPipeW ( lpnamedpipename : :: windows_sys::core::PCWSTR , lpinbuffer : *const ::core::ffi::c_void , ninbuffersize : u32 , lpoutbuffer : *mut ::core::ffi::c_void , noutbuffersize : u32 , lpbytesread : *mut u32 , ntimeout : u32 ) -> super::super::Foundation:: BOOL );
#[cfg(all(feature = "Win32_Foundation", feature = "Win32_System_IO"))]
::windows_sys::core::link ! ( "kernel32.dll""system" #[doc = "*Required features: `\"Win32_System_Pipes\"`, `\"Win32_Foundation\"`, `\"Win32_System_IO\"`*"] fn ConnectNamedPipe ( hnamedpipe : super::super::Foundation:: HANDLE , lpoverlapped : *mut super::IO:: OVERLAPPED ) -> super::super::Foundation:: BOOL );
#[cfg(all(feature = "Win32_Foundation", feature = "Win32_Security", feature = "Win32_Storage_FileSystem"))]
::windows_sys::core::link ! ( "kernel32.dll""system" #[doc = "*Required features: `\"Win32_System_Pipes\"`, `\"Win32_Foundation\"`, `\"Win32_Security\"`, `\"Win32_Storage_FileSystem\"`*"] fn CreateNamedPipeA ( lpname : :: windows_sys::core::PCSTR , dwopenmode : super::super::Storage::FileSystem:: FILE_FLAGS_AND_ATTRIBUTES , dwpipemode : NAMED_PIPE_MODE , nmaxinstances : u32 , noutbuffersize : u32 , ninbuffersize : u32 , ndefaulttimeout : u32 , lpsecurityattributes : *const super::super::Security:: SECURITY_ATTRIBUTES ) -> super::super::Foundation:: HANDLE );
#[cfg(all(feature = "Win32_Foundation", feature = "Win32_Security", feature = "Win32_Storage_FileSystem"))]
::windows_sys::core::link ! ( "kernel32.dll""system" #[doc = "*Required features: `\"Win32_System_Pipes\"`, `\"Win32_Foundation\"`, `\"Win32_Security\"`, `\"Win32_Storage_FileSystem\"`*"] fn CreateNamedPipeW ( lpname : :: windows_sys::core::PCWSTR , dwopenmode : super::super::Storage::FileSystem:: FILE_FLAGS_AND_ATTRIBUTES , dwpipemode : NAMED_PIPE_MODE , nmaxinstances : u32 , noutbuffersize : u32 , ninbuffersize : u32 , ndefaulttimeout : u32 , lpsecurityattributes : *const super::super::Security:: SECURITY_ATTRIBUTES ) -> super::super::Foundation:: HANDLE );
#[cfg(all(feature = "Win32_Foundation", feature = "Win32_Security"))]
::windows_sys::core::link ! ( "kernel32.dll""system" #[doc = "*Required features: `\"Win32_System_Pipes\"`, `\"Win32_Foundation\"`, `\"Win32_Security\"`*"] fn CreatePipe ( hreadpipe : *mut super::super::Foundation:: HANDLE , hwritepipe : *mut super::super::Foundation:: HANDLE , lppipeattributes : *const super::super::Security:: SECURITY_ATTRIBUTES , nsize : u32 ) -> super::super::Foundation:: BOOL );
#[cfg(feature = "Win32_Foundation")]
::windows_sys::core::link ! ( "kernel32.dll""system" #[doc = "*Required features: `\"Win32_System_Pipes\"`, `\"Win32_Foundation\"`*"] fn DisconnectNamedPipe ( hnamedpipe : super::super::Foundation:: HANDLE ) -> super::super::Foundation:: BOOL );
#[cfg(feature = "Win32_Foundation")]
::windows_sys::core::link ! ( "kernel32.dll""system" #[doc = "*Required features: `\"Win32_System_Pipes\"`, `\"Win32_Foundation\"`*"] fn GetNamedPipeClientComputerNameA ( pipe : super::super::Foundation:: HANDLE , clientcomputername : :: windows_sys::core::PSTR , clientcomputernamelength : u32 ) -> super::super::Foundation:: BOOL );
#[cfg(feature = "Win32_Foundation")]
::windows_sys::core::link ! ( "kernel32.dll""system" #[doc = "*Required features: `\"Win32_System_Pipes\"`, `\"Win32_Foundation\"`*"] fn GetNamedPipeClientComputerNameW ( pipe : super::super::Foundation:: HANDLE , clientcomputername : :: windows_sys::core::PWSTR , clientcomputernamelength : u32 ) -> super::super::Foundation:: BOOL );
#[cfg(feature = "Win32_Foundation")]
::windows_sys::core::link ! ( "kernel32.dll""system" #[doc = "*Required features: `\"Win32_System_Pipes\"`, `\"Win32_Foundation\"`*"] fn GetNamedPipeClientProcessId ( pipe : super::super::Foundation:: HANDLE , clientprocessid : *mut u32 ) -> super::super::Foundation:: BOOL );
#[cfg(feature = "Win32_Foundation")]
::windows_sys::core::link ! ( "kernel32.dll""system" #[doc = "*Required features: `\"Win32_System_Pipes\"`, `\"Win32_Foundation\"`*"] fn GetNamedPipeClientSessionId ( pipe : super::super::Foundation:: HANDLE , clientsessionid : *mut u32 ) -> super::super::Foundation:: BOOL );
#[cfg(feature = "Win32_Foundation")]
::windows_sys::core::link ! ( "kernel32.dll""system" #[doc = "*Required features: `\"Win32_System_Pipes\"`, `\"Win32_Foundation\"`*"] fn GetNamedPipeHandleStateA ( hnamedpipe : super::super::Foundation:: HANDLE , lpstate : *mut NAMED_PIPE_MODE , lpcurinstances : *mut u32 , lpmaxcollectioncount : *mut u32 , lpcollectdatatimeout : *mut u32 , lpusername : :: windows_sys::core::PSTR , nmaxusernamesize : u32 ) -> super::super::Foundation:: BOOL );
#[cfg(feature = "Win32_Foundation")]
::windows_sys::core::link ! ( "kernel32.dll""system" #[doc = "*Required features: `\"Win32_System_Pipes\"`, `\"Win32_Foundation\"`*"] fn GetNamedPipeHandleStateW ( hnamedpipe : super::super::Foundation:: HANDLE , lpstate : *mut NAMED_PIPE_MODE , lpcurinstances : *mut u32 , lpmaxcollectioncount : *mut u32 , lpcollectdatatimeout : *mut u32 , lpusername : :: windows_sys::core::PWSTR , nmaxusernamesize : u32 ) -> super::super::Foundation:: BOOL );
#[cfg(feature = "Win32_Foundation")]
::windows_sys::core::link ! ( "kernel32.dll""system" #[doc = "*Required features: `\"Win32_System_Pipes\"`, `\"Win32_Foundation\"`*"] fn GetNamedPipeInfo ( hnamedpipe : super::super::Foundation:: HANDLE , lpflags : *mut NAMED_PIPE_MODE , lpoutbuffersize : *mut u32 , lpinbuffersize : *mut u32 , lpmaxinstances : *mut u32 ) -> super::super::Foundation:: BOOL );
#[cfg(feature = "Win32_Foundation")]
::windows_sys::core::link ! ( "kernel32.dll""system" #[doc = "*Required features: `\"Win32_System_Pipes\"`, `\"Win32_Foundation\"`*"] fn GetNamedPipeServerProcessId ( pipe : super::super::Foundation:: HANDLE , serverprocessid : *mut u32 ) -> super::super::Foundation:: BOOL );
#[cfg(feature = "Win32_Foundation")]
::windows_sys::core::link ! ( "kernel32.dll""system" #[doc = "*Required features: `\"Win32_System_Pipes\"`, `\"Win32_Foundation\"`*"] fn GetNamedPipeServerSessionId ( pipe : super::super::Foundation:: HANDLE , serversessionid : *mut u32 ) -> super::super::Foundation:: BOOL );
#[cfg(feature = "Win32_Foundation")]
::windows_sys::core::link ! ( "advapi32.dll""system" #[doc = "*Required features: `\"Win32_System_Pipes\"`, `\"Win32_Foundation\"`*"] fn ImpersonateNamedPipeClient ( hnamedpipe : super::super::Foundation:: HANDLE ) -> super::super::Foundation:: BOOL );
#[cfg(feature = "Win32_Foundation")]
::windows_sys::core::link ! ( "kernel32.dll""system" #[doc = "*Required features: `\"Win32_System_Pipes\"`, `\"Win32_Foundation\"`*"] fn PeekNamedPipe ( hnamedpipe : super::super::Foundation:: HANDLE , lpbuffer : *mut ::core::ffi::c_void , nbuffersize : u32 , lpbytesread : *mut u32 , lptotalbytesavail : *mut u32 , lpbytesleftthismessage : *mut u32 ) -> super::super::Foundation:: BOOL );
#[cfg(feature = "Win32_Foundation")]
::windows_sys::core::link ! ( "kernel32.dll""system" #[doc = "*Required features: `\"Win32_System_Pipes\"`, `\"Win32_Foundation\"`*"] fn SetNamedPipeHandleState ( hnamedpipe : super::super::Foundation:: HANDLE , lpmode : *const NAMED_PIPE_MODE , lpmaxcollectioncount : *const u32 , lpcollectdatatimeout : *const u32 ) -> super::super::Foundation:: BOOL );
#[cfg(all(feature = "Win32_Foundation", feature = "Win32_System_IO"))]
::windows_sys::core::link ! ( "kernel32.dll""system" #[doc = "*Required features: `\"Win32_System_Pipes\"`, `\"Win32_Foundation\"`, `\"Win32_System_IO\"`*"] fn TransactNamedPipe ( hnamedpipe : super::super::Foundation:: HANDLE , lpinbuffer : *const ::core::ffi::c_void , ninbuffersize : u32 , lpoutbuffer : *mut ::core::ffi::c_void , noutbuffersize : u32 , lpbytesread : *mut u32 , lpoverlapped : *mut super::IO:: OVERLAPPED ) -> super::super::Foundation:: BOOL );
#[cfg(feature = "Win32_Foundation")]
::windows_sys::core::link ! ( "kernel32.dll""system" #[doc = "*Required features: `\"Win32_System_Pipes\"`, `\"Win32_Foundation\"`*"] fn WaitNamedPipeA ( lpnamedpipename : :: windows_sys::core::PCSTR , ntimeout : u32 ) -> super::super::Foundation:: BOOL );
#[cfg(feature = "Win32_Foundation")]
::windows_sys::core::link ! ( "kernel32.dll""system" #[doc = "*Required features: `\"Win32_System_Pipes\"`, `\"Win32_Foundation\"`*"] fn WaitNamedPipeW ( lpnamedpipename : :: windows_sys::core::PCWSTR , ntimeout : u32 ) -> super::super::Foundation:: BOOL );
#[doc = "*Required features: `\"Win32_System_Pipes\"`*"]
pub const NMPWAIT_NOWAIT: u32 = 1u32;
#[doc = "*Required features: `\"Win32_System_Pipes\"`*"]
pub const NMPWAIT_USE_DEFAULT_WAIT: u32 = 0u32;
#[doc = "*Required features: `\"Win32_System_Pipes\"`*"]
pub const NMPWAIT_WAIT_FOREVER: u32 = 4294967295u32;
#[doc = "*Required features: `\"Win32_System_Pipes\"`*"]
pub const PIPE_UNLIMITED_INSTANCES: u32 = 255u32;
#[doc = "*Required features: `\"Win32_System_Pipes\"`*"]
pub type NAMED_PIPE_MODE = u32;
#[doc = "*Required features: `\"Win32_System_Pipes\"`*"]
pub const PIPE_WAIT: NAMED_PIPE_MODE = 0u32;
#[doc = "*Required features: `\"Win32_System_Pipes\"`*"]
pub const PIPE_NOWAIT: NAMED_PIPE_MODE = 1u32;
#[doc = "*Required features: `\"Win32_System_Pipes\"`*"]
pub const PIPE_READMODE_BYTE: NAMED_PIPE_MODE = 0u32;
#[doc = "*Required features: `\"Win32_System_Pipes\"`*"]
pub const PIPE_READMODE_MESSAGE: NAMED_PIPE_MODE = 2u32;
#[doc = "*Required features: `\"Win32_System_Pipes\"`*"]
pub const PIPE_CLIENT_END: NAMED_PIPE_MODE = 0u32;
#[doc = "*Required features: `\"Win32_System_Pipes\"`*"]
pub const PIPE_SERVER_END: NAMED_PIPE_MODE = 1u32;
#[doc = "*Required features: `\"Win32_System_Pipes\"`*"]
pub const PIPE_TYPE_BYTE: NAMED_PIPE_MODE = 0u32;
#[doc = "*Required features: `\"Win32_System_Pipes\"`*"]
pub const PIPE_TYPE_MESSAGE: NAMED_PIPE_MODE = 4u32;
#[doc = "*Required features: `\"Win32_System_Pipes\"`*"]
pub const PIPE_ACCEPT_REMOTE_CLIENTS: NAMED_PIPE_MODE = 0u32;
#[doc = "*Required features: `\"Win32_System_Pipes\"`*"]
pub const PIPE_REJECT_REMOTE_CLIENTS: NAMED_PIPE_MODE = 8u32;
